#include <utility.h>
#define _DLL  

#include <ansi_c.h>
#include <cvirte.h>     
#include <userint.h>
#include "FastTest.h"
#include "rscmu200.h"
#include "rscmu2bt.h"  

static int panelHandle;
static ViReal64 nominalPower [6][4];
static ViReal64 leakagePower [6][4];
static ViReal64 peakPower [6][4];
static ViInt32 packetTiming [6][4];
static ViReal64 frequencyAccuracy [6][4];
static ViReal64 frequencyDrift [6][4];
static ViReal64 maxDriftRate [6][4];
static ViReal64 avgFreqDeviation [6][4];
static ViReal64 minFreqDeviation [6][4];
static ViReal64 maxFreqDeviation [6][4];
static ViReal64 burstOutofTolPower [6][4];
static ViReal64 burstOutofTolTiming [6][4];
static ViReal64 burstOutofTolModulation [6][4];
static ViReal64 ber_data[3];
static ViReal64 per_data[3];
static ViInt32 packet_received[3];
static ViSession vi_base;
static ViSession vi_2bt;
static ViChar error_message[512];
static ViChar buffer[512];

int UpdateTables(void);

#define FT_CHECKRET(fcal) if (rscmu_status >= 0) rscmu_status = fcal;
#define FT_CHECKERR(fcal, err) if ((rscmu_status = fcal) < 0){\
                                 RSCMUBT_close(vi_2bt);\
                                 RSCMU_close(vi_base);\
                                 sprintf (buffer, "Error: %ld, %s", rscmu_status, err);\
                                 MessagePopup ("Error", buffer);\
                                 SetWaitCursor (0);\
                                 return 0;\
                                 }
                                 
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "FastTest.uir", PANEL)) < 0)
        return -1;
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    return 0;
}

int CVICALLBACK Run_Click (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32     lval;
    ViStatus    rscmu_status = 0;
    ViInt32     signalling_state; 
    ViInt32     i;
    ViInt32     idx;
    
    switch (event)
        {
        case EVENT_COMMIT:
            SetWaitCursor (1);
            GetCtrlVal (panelHandle, PANEL_RESSOURCE_STRING, buffer);
            
            rscmu_status = RSCMU_init (buffer, VI_TRUE, VI_FALSE, &vi_base);
            if (rscmu_status < 0)
            {
                MessagePopup ("Error", "Could not open base session!");
                return 0;
            }   
            FT_CHECKRET (RSCMU_Base_SystRemAddrSec(vi_base, 1, RSCMU_BLUETOOTH_SIG, error_message));
            FT_CHECKRET (RSCMUBT_Init_BT_Sig (vi_base, &vi_2bt, error_message));
            GetCtrlVal (panelHandle, PANEL_DUT, &idx);  
            GetLabelFromIndex (panelHandle, PANEL_DUT, idx, buffer);
            FT_CHECKRET (RSCMUBT_Sig_ConfMsigPagTarg(vi_2bt, buffer, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerTsetDel(vi_2bt, 1, RSCMU_OFF, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfMsigPagPsrm(vi_2bt, 1, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfMsigPagRsin(vi_2bt, RSCMU_ON, error_message));
            
            FT_CHECKRET(RSCMUBT_Sig_ProcSignAct(vi_2bt, 3, error_message));

            do{
                
                rscmu_status = RSCMUBT_Sig_SensXSignStat(vi_2bt, &signalling_state, error_message);
                if (signalling_state == 0)
                {
                    rscmu_status = -1;
                    strcpy(error_message, "Could not establish the connection.");     
                }
            }while ((signalling_state != 4) && (rscmu_status >=0));
            GetCtrlVal (panelHandle, PANEL_TX_BURST, &lval); 
            FT_CHECKRET (RSCMUBT_Sig_ConfPowMprContStat(vi_2bt, lval, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfPowMprContRep(vi_2bt, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfPowMprMmod(vi_2bt, 0, error_message));
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodTmty(vi_2bt, 0, 1, error_message));
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodHsch(vi_2bt, 0, 5, error_message));
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodTxtPtyp(vi_2bt, 0, 2, error_message));
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodTxtLots(vi_2bt, 0, 2, 339, error_message));
            
        
            //Tx Channel 0 
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodTxtChan(vi_2bt, 0, 0, error_message));
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodTxtPatt(vi_2bt, 1, 3, error_message));    
            FT_CHECKRET (RSCMUBT_Sig_InitPowMpr(vi_2bt, error_message));
            for (i=0;i<=3;i++)
                FT_CHECKRET (RSCMUBT_Sig_FetcPowMpr(vi_2bt, i, &nominalPower[0][i], &leakagePower[0][i], &peakPower[0][i], &packetTiming[0][i], &frequencyAccuracy[0][i], &frequencyDrift[0][i], &maxDriftRate[0][i], &avgFreqDeviation[0][i], &minFreqDeviation[0][i], &maxFreqDeviation[0][i], &burstOutofTolPower[0][i], &burstOutofTolTiming[0][i], &burstOutofTolModulation[0][i], error_message));
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodTxtPatt(vi_2bt, 1, 4, error_message));
            FT_CHECKRET (RSCMUBT_Sig_InitPowMpr(vi_2bt, error_message));
            for (i=0;i<=3;i++)
                FT_CHECKRET (RSCMUBT_Sig_FetcPowMpr(vi_2bt, i, &nominalPower[1][i], &leakagePower[1][i], &peakPower[1][i], &packetTiming[1][i], &frequencyAccuracy[1][i], &frequencyDrift[1][i], &maxDriftRate[1][i], &avgFreqDeviation[1][i], &minFreqDeviation[1][i], &maxFreqDeviation[1][i], &burstOutofTolPower[1][i], &burstOutofTolTiming[1][i], &burstOutofTolModulation[1][i], error_message));
            
            //Tx Channel 39 
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodTxtChan (vi_2bt, RSCMU_CONF, 39, error_message));
            FT_CHECKRET (RSCMUBT_Sig_InitPowMpr(vi_2bt, error_message));
            for (i=0;i<=3;i++)
                FT_CHECKRET (RSCMUBT_Sig_FetcPowMpr(vi_2bt, i, &nominalPower[3][i], &leakagePower[3][i], &peakPower[3][i], &packetTiming[3][i], &frequencyAccuracy[3][i], &frequencyDrift[3][i], &maxDriftRate[3][i], &avgFreqDeviation[3][i], &minFreqDeviation[3][i], &maxFreqDeviation[3][i], &burstOutofTolPower[3][i], &burstOutofTolTiming[3][i], &burstOutofTolModulation[3][i], error_message));
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodTxtPatt(vi_2bt, 1, 3, error_message));
            FT_CHECKRET (RSCMUBT_Sig_InitPowMpr(vi_2bt, error_message));
            for (i=0;i<=3;i++)
                FT_CHECKRET (RSCMUBT_Sig_FetcPowMpr(vi_2bt, i, &nominalPower[2][i], &leakagePower[2][i], &peakPower[2][i], &packetTiming[2][i], &frequencyAccuracy[2][i], &frequencyDrift[2][i], &maxDriftRate[2][i], &avgFreqDeviation[2][i], &minFreqDeviation[2][i], &maxFreqDeviation[2][i], &burstOutofTolPower[2][i], &burstOutofTolTiming[2][i], &burstOutofTolModulation[2][i], error_message));
        
            //Tx Channel 78 
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodTxtChan (vi_2bt, RSCMU_CONF, 78, error_message));
            FT_CHECKRET (RSCMUBT_Sig_InitPowMpr(vi_2bt, error_message));
            for (i=0;i<=3;i++)
                FT_CHECKRET (RSCMUBT_Sig_FetcPowMpr(vi_2bt, i, &nominalPower[4][i], &leakagePower[4][i], &peakPower[4][i], &packetTiming[4][i], &frequencyAccuracy[4][i], &frequencyDrift[4][i], &maxDriftRate[4][i], &avgFreqDeviation[4][i], &minFreqDeviation[4][i], &maxFreqDeviation[4][i], &burstOutofTolPower[4][i], &burstOutofTolTiming[4][i], &burstOutofTolModulation[4][i], error_message));
            FT_CHECKRET (RSCMUBT_Sig_SsigTmodTxtPatt(vi_2bt, 1, 4, error_message));
            FT_CHECKRET (RSCMUBT_Sig_InitPowMpr(vi_2bt, error_message));
            for (i=0;i<=3;i++)
                FT_CHECKRET (RSCMUBT_Sig_FetcPowMpr(vi_2bt, i, &nominalPower[5][i], &leakagePower[5][i], &peakPower[5][i], &packetTiming[5][i], &frequencyAccuracy[5][i], &frequencyDrift[5][i], &maxDriftRate[5][i], &avgFreqDeviation[5][i], &minFreqDeviation[5][i], &maxFreqDeviation[5][i], &burstOutofTolPower[5][i], &burstOutofTolTiming[5][i], &burstOutofTolModulation[5][i], error_message));
        
            FT_CHECKRET (RSCMUBT_Sig_AborPowMpr(vi_2bt, error_message));
            
            //Recevier test
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerLev(vi_2bt, 1, -70.0, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerTsetPtyp(vi_2bt, 1, 2, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerTsetLots(vi_2bt, 1, 2, 339, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerTsetHsch(vi_2bt, 1, 5, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerTsetPatt(vi_2bt, 1, 1, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerContRep(vi_2bt, 1, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, error_message));
            GetCtrlVal (panelHandle, PANEL_RX_BURST, &lval);  
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerContStat(vi_2bt, 1, lval, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerTset(vi_2bt, 1, error_message));

            //BER Channel 0
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerTsetChan(vi_2bt, 1, 0, 78, error_message));
            FT_CHECKRET (RSCMUBT_Sig_InitRxqBer(vi_2bt, error_message));
            FT_CHECKRET (RSCMUBT_Sig_FetcRxqBer(vi_2bt, &ber_data[0], &per_data[0], &packet_received[0], error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerTsetChan(vi_2bt, 1, 39, 0, error_message));
            FT_CHECKRET (RSCMUBT_Sig_InitRxqBer(vi_2bt, error_message));
            FT_CHECKRET (RSCMUBT_Sig_FetcRxqBer(vi_2bt, &ber_data[1], &per_data[1], &packet_received[1], error_message));
            FT_CHECKRET (RSCMUBT_Sig_ConfRxqBerTsetChan(vi_2bt, 1, 78, 0, error_message));
            FT_CHECKRET (RSCMUBT_Sig_InitRxqBer(vi_2bt, error_message));
            FT_CHECKRET (RSCMUBT_Sig_FetcRxqBer(vi_2bt, &ber_data[2], &per_data[2], &packet_received[2], error_message));
            FT_CHECKRET (RSCMUBT_Sig_ProcSignAct(vi_2bt, 6, error_message));
            FT_CHECKRET (RSCMUBT_Sig_ProcSignAct(vi_2bt, 5, error_message));
            RSCMUBT_close(vi_2bt);
            RSCMU_close(vi_base);
            if (rscmu_status < 0)
            {
                sprintf (buffer, "Error: %ld, %s", rscmu_status, error_message);
                MessagePopup ("Error", buffer); 
            }
            
            UpdateTables();
            break;
        }
    SetWaitCursor (0);
    return 0;
}

int CVICALLBACK ChannelSelect_IndexChanged (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            UpdateTables();
            break;
        }
    return 0;
}

int CVICALLBACK Scan_Click (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32     nb_of_devices;
    ViInt32     i, delay_step, time;
    ViStatus    rscmu_status = 0;
    ViInt16     garbage, stb;
    ViChar      *p_token;
    ViChar      p_tmp[512];
    
    switch (event)
        {
        case EVENT_COMMIT:
            SetWaitCursor (1); 
            GetCtrlVal (panelHandle, PANEL_RESSOURCE_STRING, buffer);
            rscmu_status = RSCMU_init (buffer, VI_TRUE, VI_FALSE, &vi_base);
            if (rscmu_status < 0)
            {
                MessagePopup ("Error", "Could not open base session!");
                SetWaitCursor (0); 
                return 0;
            }   
            FT_CHECKRET (RSCMU_Base_SystRemAddrSec(vi_base, 1, RSCMU_BLUETOOTH_SIG, error_message));
            FT_CHECKRET (RSCMUBT_Init_BT_Sig(vi_base, &vi_2bt, error_message));

        //configure status subsystem to generate SRQ
            FT_CHECKERR (viWrite(vi_base, "*SRE 128", 8, 0), "Unable to set status registers!");
            
            FT_CHECKRET (RSCMU_Base_StatOperCmuSumCmuEnab(vi_base, RSCMU_SUM1, 2, 8, error_message));
            FT_CHECKRET (RSCMU_Base_StatOperCmuSumEnab(vi_base, RSCMU_SUM1, 2, error_message));
            FT_CHECKRET (RSCMU_Base_StatOperEnab(vi_base, 511, error_message));

#if defined SRQ_ENABLED
            FT_CHECKRET (viEnableEvent(vi_2bt, VI_EVENT_SERVICE_REQ, VI_QUEUE, 0));
#endif

            //start inquiry
            FT_CHECKRET (RSCMUBT_Sig_ProcSignAct(vi_2bt, 0, error_message));

#if defined SRQ_ENABLED
            FT_CHECKERR (viWaitOnEvent(vi_2bt, VI_ALL_ENABLED_EVENTS, 20000, 0, 0), "No DUT found!");
          
            FT_CHECKRET (viDiscardEvents(vi_2bt, VI_ALL_ENABLED_EVENTS, VI_QUEUE));

            FT_CHECKRET (viReadSTB(vi_base, &garbage));
#else
            delay_step = 10;
            time = 0;
            do
            {
                FT_CHECKRET (viReadSTB(vi_base, &stb));
                Delay (delay_step * 0.001);
                time += delay_step;
            } while (!((stb & 0x40) || (time > 20000)));
#endif
       
            FT_CHECKRET (RSCMUBT_Sig_FetcSignPtar(vi_2bt, &nb_of_devices, buffer, error_message));
            ClearListCtrl (panelHandle, PANEL_DUT);
            i=0;

            p_token = strstr (buffer, "BD01");
            if (p_token != VI_NULL)
            {
                p_token = strtok (p_token,",");
                p_token = strtok (NULL, ",");
                sscanf (p_token, "\"%[^\"]\"", p_tmp);
                InsertListItem (panelHandle, PANEL_DUT, -1, p_tmp, i);
                for (i=1;i<nb_of_devices;i++)
                {
                    p_token = strtok (NULL, ",");
                    p_token = strtok (NULL, ",");
                    sscanf (p_token, "\"%[^\"]\"", p_tmp); 
                    InsertListItem (panelHandle, PANEL_DUT, -1, p_tmp, i);
                }   
            }
            
            //FT_CHECKRET (RSCMUBT_Sig_ProcSignAct(vi_2bt, 1, error_message));
            RSCMUBT_close(vi_2bt);
            RSCMU_close(vi_base);
            if (rscmu_status < 0)
            {
                sprintf (buffer, "Error: %ld, %s", rscmu_status, error_message);
                MessagePopup ("Error", buffer); 
            }
             SetWaitCursor (0);    
            break;
        }
    return 0;
}

int CVICALLBACK Panel_Close (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_GOT_FOCUS:
        case EVENT_LOST_FOCUS:
            break;
        case EVENT_CLOSE:
            QuitUserInterface (0);
            break;
        }
    return 0;
}

int UpdateTables ()
{
    long    channel_index;
    Rect    range;
    //ViChar  row_str [4] [20];
    ViChar  **row_str;
    ViInt32 i;
    
    //Update transmitter table
    GetCtrlVal (panelHandle, PANEL_CHANNEL_SELECT, &channel_index); 
    //Cells are indexed from 1
    range.top = 1;
    range.left = 2;
    range.width = 4;
    range.height = 1;

    
    row_str=malloc(sizeof (char *)*4);
    for (i=0;i<4;i++)
        row_str[i]=malloc(sizeof(char)*20);
        
    for (i = 0; i <= 3; i++)
    {
        if (nominalPower[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", nominalPower[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (leakagePower[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", leakagePower[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (peakPower[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", peakPower[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (packetTiming[channel_index][i] != RSCMU_NAN_VI_INT32)
            sprintf (row_str[i], "%ld", packetTiming[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (frequencyAccuracy[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", frequencyAccuracy[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (frequencyDrift[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", frequencyDrift[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (maxDriftRate[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", maxDriftRate[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (avgFreqDeviation[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", avgFreqDeviation[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (minFreqDeviation[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", minFreqDeviation[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (maxFreqDeviation[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", maxFreqDeviation[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (burstOutofTolPower[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", burstOutofTolPower[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (burstOutofTolTiming[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", burstOutofTolTiming[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
    range.top++;
    for (i = 0; i <= 3; i++)
    {
        if (burstOutofTolModulation[channel_index][i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", burstOutofTolModulation[channel_index][i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_TABLE, range, row_str, VAL_ROW_MAJOR);
 
    //Update receiver table
    range.top = 1;
    range.left = 2;
    range.width = 1;
    range.height = 3;
    for (i = 0; i < 3; i++)
    {
        if (ber_data[i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", ber_data[i]);
        else
            sprintf (row_str[i], "-.-");
    }
    SetTableCellRangeVals (panelHandle, PANEL_REC_LIST, range, row_str, VAL_COLUMN_MAJOR);
    for (i = 0; i < 3; i++)
    {
        if (per_data[i] != RSCMU_NAN)
            sprintf (row_str[i], "%.3lf", per_data[i]);
        else
            sprintf (row_str[i], "-.-");
    }
    range.left++;
    SetTableCellRangeVals (panelHandle, PANEL_REC_LIST, range, row_str, VAL_COLUMN_MAJOR);
    for (i = 0; i < 3; i++)
    {
        if (packet_received[i] != RSCMU_NAN_VI_INT32)
            sprintf (row_str[i], "%ld", packet_received[i]);
        else
            sprintf (row_str[i], "-.-");
    }
    range.left++;
    SetTableCellRangeVals (panelHandle, PANEL_REC_LIST, range, row_str, VAL_COLUMN_MAJOR);
   
    for (i=0;i<4;i++)
       free(row_str[i]);
    free (row_str);   
    
    return 0;
}


